#!/bin/bash

base="$( cd "$(dirname "${BASH_SOURCE[0]}")" && pwd )"

# Allow this script to be run only by ubuntu-it
if [[ "$(whoami)" != "ubuntu-it" ]]; then
    echo "Error: this script must be executed by the ubuntu-it user"
    exit 1
fi

chronic_sh (){
    local pipe pipe_q
    pipe=$(mktemp -p "${TMPDIR:-/tmp}" chronic_sh.XXXXXXXXXX)
    printf -v pipe_q '%q' "$pipe"
    trap 'rm -fv '"$pipe_q" RETURN
    # one can't open a reading fd and a writing fd without blocking, because
    # they want to already have something on the other side of the pipe.
    # the temporary fd 5 will be that something.
    exec 5<>"$pipe" # hack
    exec 3>"$pipe"  # writing fd
    exec 4<"$pipe"  # reading fd
    exec 5>&-       # end hack
    rm "$pipe"

    local ret=0
    "$@" >&3 2>&3 || ret=$?
    if [ "$ret" -ne 0 ] ; then
        exec 3>&-
        cat <&4-
        return $ret
    fi
}

# Try to deploy all the websites
for script in "${base}/scripts/"*; do
    if [[ ! -x "${script}" ]]; then
        continue
    fi

    # Create a temporary build directory
    build_dir="$(mktemp -d deploy-XXXXXXXXX -p /tmp)"

    # Switch to the temporary build directory
    cd "${build_dir}"
    export HOME="${build_dir}"
    TMPDIR="$HOME"
    output="$(mktemp -p "${TMPDIR:-/tmp}" deploy_output.XXXXXXXX)"

    # Execute the deploy script
    # Capture the output and print it only if the script fails
    "$script" > "$output"
    if [[ $? -ne 0 ]]; then
        echo "Deploy script $(basename "${script}") failed!"
        echo "Script output"
        echo "============="
        cat "${output}"
        echo
    fi

    # Restore the old home
    cd "${base}"
    export HOME="${base}"

    # Remove the temporary directory
    rm -rf "${build_dir}"

done
